@@ -4,8 +4,11 @@ from .decorators import Plugin
 def invert_dict(d: dict):
     inverse_dict = {}
     for k, v in d.items():
-        for item in v:
-            inverse_dict[item] = k
+        if isinstance(v, list):
+            for item in v:
+                inverse_dict[item] = k
+        else:
+            inverse_dict[v] = k
     return inverse_dict
 
 
